import com.atlassian.jira.component.ComponentAccessor
import com.atlassian.jira.issue.search.SearchRequestManager
import com.atlassian.query.clause.*


if (!binding.variables.containsKey("automationTriggerJiraJqlExpression")) {
    automationTriggerJiraJqlExpression = null
}
if (!binding.variables.containsKey("searchRequest")) {
    searchRequest = null
}

issueNavigator = new IssueNavigatorTool(defaultSearchRequest: searchRequest, automationTriggerJiraJqlExpression: automationTriggerJiraJqlExpression)

/**
 * Tool to export dynamic column configurations from Issue Navigator.
 */
public class IssueNavigatorTool {
    def searchRequestManager = ComponentAccessor.getOSGiComponentInstanceOfType(SearchRequestManager.class)

    def defaultSearchRequest
    def automationTriggerJiraJqlExpression

    def getSearchRequest() {
        def searchRequest
        // if automation is used, find the search request (saved filter) object based on the automation trigger's JQL
        def jql = automationTriggerJiraJqlExpression
        if (jql != null) {
            def filterIdOrName = parseFilterIdOrNameFromJql(jql)
            if (filterIdOrName != null) {
                if (filterIdOrName.isInteger()) {
                    // find by ID
                    searchRequest = searchRequestManager.getSearchRequestById(Long.parseLong(filterIdOrName))
                }
                if (searchRequest == null) {
                    // find by name
                    def searchRequestsFound = searchRequestManager.findByNameIgnoreCase(filterIdOrName)
                    if (!searchRequestsFound.empty) {
                        searchRequest = searchRequestsFound.get(0)
                    }
                }
            }
        }

        return (searchRequest != null) ? searchRequest : defaultSearchRequest
    }

    /**
     * Parses a saved filter's name or ID from a JQL query string (in the format: 'filter = 12' or 'filter = "My open bugs"').
     * @return filter id or name without quotation marks or apostrophes, or null if wasn't found.
     */
    private parseFilterIdOrNameFromJql(String jql) {
        // parse JQL
        def clazz = ComponentAccessor.class.classLoader.loadClass("com.atlassian.jira.jql.parser.JqlQueryParser")
        def jqlQueryParser = ComponentAccessor.getOSGiComponentInstanceOfType(clazz)
        def query = jqlQueryParser.parseQuery(jql)

        // search for the "filter =" clause
        def filterIdOrName = query.whereClause.accept(new ClauseVisitor<String>() {
            String visit(AndClause andClause) {
                visitSubClauses(andClause.clauses)
            }

            String visit(NotClause notClause) {
                visitSubClauses(notClause.clauses)
            }

            String visit(OrClause orClause) {
                visitSubClauses(orClause.clauses)
            }

            String visit(TerminalClause clause) {
                return (("filter".equals(clause.name)) && ("=".equals(clause.operator.displayString))) ? ((clause.operand.longValue != null) ? clause.operand.longValue : clause.operand.stringValue) : null
            }

            String visit(WasClause clause) {
                return null
            }

            String visit(ChangedClause clause) {
                return null
            }

            private String visitSubClauses(def clauses) {
                for(clause in clauses) {
                    def filterIdOrName = visit(clause)
                    if(filterIdOrName != null) {
                        return filterIdOrName
                    }
                }
                return null
            }
        })

        return filterIdOrName
    }
}
